<?php
namespace Odhen\API\Service;

use \SimpleXMLElement;

class PagerDigital {
	protected $entityManager;
	
	public function __construct(
		\Doctrine\ORM\EntityManager $entityManager
	){
        $this->entityManager = $entityManager;
	}
    public function chamadaAPIPagerDigital($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $CDURLPGEDIG){
        try {
            self::LogPagerDig('--------------------------------------------------');
            self::LogPagerDig('INICIO');
            $Result=array('erro'=>'Atenção! Erro na conexão com o Pager Digital. Não será possível acompanhar o status do pedido.');

            if(empty($CDURLPGEDIG)){
                self::LogPagerDig('Endereço da API PagerDigital não parametrizado.');
                return false;
            }
            if(substr($CDURLPGEDIG, -1) != '/' ) {
                $CDURLPGEDIG = $CDURLPGEDIG . '/';
            }
            $CDURLPGEDIG = $CDURLPGEDIG . 'order';

            self::LogPagerDig('P_CDFILIAL = '.$CDFILIAL.' P_CDCAIXA = '.$CDCAIXA.' P_NRSEQVENDA = '.$NRSEQVENDA);

            $params = array(
                'P_CDFILIAL'   => $CDFILIAL,
                'P_CDCAIXA'    => $CDCAIXA,
                'P_NRSEQVENDA' => $NRSEQVENDA,
            );
            $arrDadosVendaPagerDig = $this->entityManager->getConnection()->fetchAll("GET_DADOS_VENDA_PAPERDIG", $params);
            if(!empty($arrDadosVendaPagerDig)){
                $dom = new \DOMDocument('1.0', 'utf-8');
                $dom->formatOutput = true;
                
                $root = $dom->createElement('root');
                $data = $dom->createElement('data');

                $DTVENDA = new \DateTime($arrDadosVendaPagerDig[0]['DTVENDA']);
                $DTVENDA = $DTVENDA->format('d/m/Y H:i:s');
                $params = [
                    'showTimer'               => $arrDadosVendaPagerDig[0]['IDUSATMPMEDPGE'] == 'S'?'true':'false',
                    'showDelayedOrderMessage' => $arrDadosVendaPagerDig[0]['IDMSGATRASOPGE'] == 'S'?'true':'false',
                    'clientLogoImg'           => $arrDadosVendaPagerDig[0]['BS64IMGPGEDIG'],
                    'estimatedTime'           => $arrDadosVendaPagerDig[0]['NRTEMPOPRODUCAO'],
                    'saleAddress'             => $arrDadosVendaPagerDig[0]['DSENDPGEDIG'],
                    'saleSeq'                 => $arrDadosVendaPagerDig[0]['NRNOTAFISCALCE'],
                    'posOpeningDate'          => $DTVENDA,
                    'totalValue'              => floatval($arrDadosVendaPagerDig[0]['VRTOTVENDA'])
                ];
                foreach ($params as $key => $value){
                    $item = $dom->createElement($key);
                    $item->nodeValue = $value;
                    $data->appendChild($item);
                }
                $products = $dom->createElement('products');
                foreach ($arrDadosVendaPagerDig as $produto){
                    $params = [
                        'productCode' => $produto['CDPRODUTO'],
                        'productName' => $produto['NMPRODUTO'],
                        'quantity'    => $produto['QTPRODVEND'],
                        'unitValue'   => floatval($produto['VRUNITVEND']),
                        'totalValue'  => floatval($produto['VRTOTPROD'])
                    ];
                    $prod = $dom->createElement('prod');
                    foreach($params as $idx => $value) {
                        $item = $dom->createElement($idx);
                        $item->nodeValue = $value;
                        $prod->appendChild($item);
                    }
                    $products->appendChild($prod);
                }
                $data->appendChild($products);
                $root->appendChild($data);
                $dom->appendChild($root);
                $stResult = self::RequisicaoHTTPS(
                    $CDURLPGEDIG,
                    $dom->saveXML(),
                    'POST',
                    'Basic dGVrbmlzYTohQEFva3ojWi1vMTVOa3QlK1t9djIm',
                    '10',
                    'xml'
                );
                if($stResult){
                    if(isset($stResult['data'])){
                        $stResult=$stResult['data'];
                        if(isset($stResult['url']) && isset($stResult['orderId']) ){
                            $Result=array();
                            $Result['CDURLQRCODEPGE'] = $stResult['url'];
                            $Result['CDPEDPGEDIG'] = $stResult['orderId'];
                        }
                    }
                }
                return $Result;
            } else {
                self::LogPagerDig('Os dados da venda não foram encontrados');
            }
            self::LogPagerDig('FIM');
        } catch(\Exception $e){
            self::LogPagerDig('Erro API PagerDigital: '.$e->getMessage());
            $Result=array('erro'=>'Atenção! Erro na conexão com o Pager Digital. Não será possível acompanhar o status do pedido.');
        }
        return $Result;
    }
    public function RequisicaoHTTPS($url, $params="", $method="POST", $authorization='', $timeOut=20, $contentType='xml'  ){
        $header = array();
        if(!empty($authorization)){
            array_push($header,"Authorization: ".$authorization);   
        }
        if($contentType =='xml'){
            array_push($header,"Content-Type: text/xml");
        }

        if(empty($timeOut)){
            $timeOut = 20;
        }

        $curl = curl_init();
        self::LogPagerDig('URL: '.$url);
		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
		curl_setopt($curl, CURLOPT_HTTPHEADER, $header);

        self::LogPagerDig('PARAMETERS: '. preg_replace( '/[\n,  ]/', "", $params ));
        curl_setopt($curl, CURLOPT_POSTFIELDS,$params);

        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_ENCODING,'');
        curl_setopt($curl, CURLOPT_MAXREDIRS,10);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_HTTP_VERSION , CURL_HTTP_VERSION_1_1);

        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        //self::LogPagerDig('TIMEOUT: '.$timeOut);
        curl_setopt($curl, CURLOPT_TIMEOUT, $timeOut);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, $timeOut);

        $curl_response = curl_exec($curl);
        self::LogPagerDig('curl Executada');
        $err    = curl_errno($curl);
        $errmsg = curl_error($curl);
        $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);
        self::LogPagerDig('HTTPCODE: '.$httpcode);
        if ($httpcode ==='404' || $httpcode === 404){
            self::LogPagerDig('Erro 404');
            throw new \Exception("<br>Erro 404. Servidor não encontrado. Verifique se a URL parametrizada esta correta.");
        } else {
            self::LogPagerDig('RESPONSE: '.$curl_response);
        }
        if ($err){
            if ($err === 28){
                if(empty($errmsg)){
                    $errmsg = "<br>N&atilde;o foi poss&iacute;vel estabelecer uma conex&atilde;o com o servidor.";
                }
            } else {
                $errmsg = "<br>Ocorreu um erro na chamada do servidor:<br><br>" .$err.' '.$errmsg;
            }
            if(empty($errmsg)){
                $errmsg = "<br>Ocorreu um erro na chamada do servidor.";
            }
            throw new \Exception($errmsg);
        }
        if (empty($curl_response)){
            self::LogPagerDig('empty curl_response');
            throw new \Exception("<br>N&atilde;o houve retorno do servidor.");
        }

        $result = self::XmlToJson($curl_response);
        $result = json_decode($result, true);
    
        return $result;
    }

    public function XmlToJson ($xml) {
        self::LogPagerDig('XmlToJson');
        $xml = str_replace(array("\n", "\r", "\t"), '', $xml);
        $xml = trim(str_replace('"', "'", $xml));
        $simpleXml = simplexml_load_string($xml);
        $json = json_encode($simpleXml);
        self::LogPagerDig($json);
        return $json;
    }

    function ArrayToXml($array, $xml = null){
        $boprimeira = false;
        if($xml == null){
            $xml = new SimpleXMLElement('<root/>');
            $boprimeira = true;
            self::LogPagerDig('ArrayToXml');
        }        
        foreach ($array as $key => $value) {
            if(is_int($key) && is_array($value)){
                self::ArrayToXml($value, $xml);
            } else {
                if(is_int($key)){
                    if($key===0){
                        $key = '0';
                    } else {
                        $key = strval($key);
                    }
                }
                if(is_bool($key)){
                    if($key){
                        $key = 'true';
                    }else {
                        $key = 'false';
                    }
                }
                if(is_array($value)){
                    $label = $xml->addChild($key);
                    self::ArrayToXml($value, $label);
                }
                else {
                    $xml->addChild($key, $value);
                }
            }
        }
        if($boprimeira){
            //self::LogPagerDig($xml->asXml());
        }
        return $xml->asXml();
    }

    public function LogPagerDig($texto){
        return self::Log('LogPagerDigital',$texto);
    }

    public function Log($tipo,$texto){
        $paths = ['LOG/', $tipo.'/', (new \DateTime('NOW', new \DateTimeZone('America/Bahia')))->format('m-Y')];
        $logPath = dirname(__DIR__) . '/../../../../../../../';
        foreach ($paths as $path){
            $logPath = $logPath . $path;
            if (!is_dir($logPath)){
                mkdir($logPath, 0700);
            }
        }
        $logPath = $logPath .'/'. (new \DateTime('NOW', new \DateTimeZone('America/Bahia')))->format('d-m-Y') . '.txt';
        $horaAtual = (new \DateTime('NOW', new \DateTimeZone('America/Bahia')))->format('d/m/Y H:i:s');
        file_put_contents($logPath, $horaAtual . ' - ' . $texto . "\r\n", FILE_APPEND);
    }
}